import { domainDescriptions, domainOf, throwParseError } from "@ark/util";
import { Disjoint } from "../shared/disjoint.js";
import { implementNode } from "../shared/implement.js";
import { writeUnsupportedJsonSchemaTypeMessage } from "../shared/jsonSchema.js";
import { InternalBasis } from "./basis.js";
const implementation = implementNode({
    kind: "domain",
    hasAssociatedError: true,
    collapsibleKey: "domain",
    keys: {
        domain: {}
    },
    normalize: schema => typeof schema === "string" ? { domain: schema } : schema,
    defaults: {
        description: node => domainDescriptions[node.domain],
        actual: data => domainDescriptions[domainOf(data)]
    },
    intersections: {
        domain: (l, r) => Disjoint.init("domain", l, r)
    }
});
export class DomainNode extends InternalBasis {
    traverseAllows = data => domainOf(data) === this.domain;
    compiledCondition = this.domain === "object" ?
        `((typeof data === "object" && data !== null) || typeof data === "function")`
        : `typeof data === "${this.domain}"`;
    compiledNegation = this.domain === "object" ?
        `((typeof data !== "object" || data === null) && typeof data !== "function")`
        : `typeof data !== "${this.domain}"`;
    expression = this.domain;
    get shortDescription() {
        return domainDescriptions[this.domain];
    }
    innerToJsonSchema() {
        if (this.domain === "bigint" || this.domain === "symbol")
            return throwParseError(writeUnsupportedJsonSchemaTypeMessage(this.domain));
        return {
            type: this.domain
        };
    }
}
export const Domain = {
    implementation,
    Node: DomainNode
};
